import sys

command_list = {

    'az_login':'az login',
    'az_login_check': 'az group list',
    'aks_list': 'az aks list --query \"[*].{Name:name,ResourceGroup:resourceGroup}\" -o tsv',
    'account_list' : 'az account list --query \"[*].{Name:name,Default:isDefault}\" -o tsv',
    'account': 'az account show --query \"name\" -o tsv',
    'dashboard_pod' : 'kubectl get pods -n kube-system --output name --selector k8s-app=kubernetes-dashboard',

}

common = {

    'dashboard_pod' : 'kubectl get pods -n kube-system --output name --selector k8s-app=kubernetes-dashboard',

}

azure = {

    'login':'az login',
    'login_check': 'az group list',
    'k8s_list': 'az aks list --query \"[*].{Name:name,ResourceGroup:resourceGroup}\" -o tsv',
    'account_list' : 'az account list --query \"[*].{Name:name,Default:isDefault}\" -o tsv',
    'account': 'az account show --query \"name\" -o tsv',

}

google = {

    'login':'gcloud auth login',
    'login_check': 'gcloud projects list',
    'k8s_list': 'gcloud container clusters list --format=\"value(selfLink.scope(clusters),selfLink.scope(projects).segment(0))\"',

}


def get_cmd(cmd,cloud=common):

    if cloud == 'azure':

        cloud_provider = azure

    elif cloud == 'google':

        cloud_provider = google

    else:

        cloud_provider = common

    if not cmd in cloud_provider:
        print(cmd)
        sys.exit('\nNo such azure command, check command_list dict')

    return cloud_provider[cmd]



def get_credentials(cloud,cluster_name,resource_group):
    

    if cloud == 'azure':

        get_kubeconfig_cmd = "az aks get-credentials -n {} -g {}".format(cluster_name,resource_group)
    
    else:

       get_kubeconfig_cmd = "gcloud container clusters get-credentials {}".format(cluster_name)
       
       print(get_kubeconfig_cmd)

    return get_kubeconfig_cmd


import sys
import os
import shlex
from halo import Halo
import colorama
import yaml
import time
from socket import socket
import subprocess
import threading
from subprocess import PIPE, Popen, call, check_output
import webbrowser
from kubernetes import client, config
from prettytable import PrettyTable

from config._cmds import get_cmd, get_credentials
from config._config import get_k8s_config

K8S_CONFIG = get_k8s_config()

'''
def checkInternet():
    
    
    This is to test the internet connection for your machine.
    Tries to connect to login.microsoftonline.com.
    
    
    try:
        # see if we can resolve the host name -- tells us if there is
        # a DNS listening
        host = socket.gethostbyname('login.microsoftonline.com')
        # connect to the host -- tells us if the host is actually
        # reachable
        socket.create_connection((host, 443), 5)
        return True

    except:
        pass
        return False

'''


def login(cloud,spinner):
    
    '''
    Checks if there is valid azure login session, if not
    make an azure login using azure cli --> az login

    Return:
            True for successful login

    '''
    # Check if we have valid azure cli session
    try:
        process = Popen(shlex.split(get_cmd('login_check',cloud)), stdout=PIPE, stderr=PIPE)
        process.communicate(timeout=5)
        rc = process.wait()

              # Doing Azure login
        while rc:
            try:
                process = Popen(shlex.split(get_cmd('login',cloud)), stdout=PIPE, stderr=PIPE)
                process.communicate()    # execute it, the output goes to the stdout
                rc = process.wait()
            except Exception as e:
                spinner.fail(colorama.Fore.RED + 'Login failed, try doing login with \"az login\"..')
                sys.exit(e)

        #default_subscription = (check_output(get_cmd('account',cloud),shell=True).decode('utf-8')).rstrip()
       # spinner.succeed(colorama.Fore.GREEN + 'Login successded --> ({})'.format(default_subscription))
        spinner.succeed(colorama.Fore.GREEN + 'Login successded')

        return True

    except subprocess.TimeoutExpired as e :
        (colorama.Fore.RED + 'Timeout of Azure Login {}!'.format(e))
        return False
    

    return False
    

def _config_loader():


    with open(K8S_CONFIG) as stream:

        config = yaml.safe_load(stream)

        if config == None:
            return False

        return config


def _isExist(cluster_name):
    
    '''
    
    Read ~/.kube/config to check if the provided cluster name is already configured under ~/.kube/config

    :param cluster_name: Name of cluster to check against.
    :return: True or False 

    '''
    config = _config_loader()

    if config:

        for cluster in config.get('clusters', []):                
                if (cluster['name'] == cluster_name):
                    return True 

    return False



def get_current_context():
    
    '''
    Read ~/.kube/config to get the current context from the merged kubeconfig --> 'kubectl config current-context'

    return: current_context 
    '''

    config = _config_loader()
    if config:
        return config.get('current-context')


def get_K8SList(cloud,output=False):
    
    '''
    To get the list of AKS Cluster for the default Azure subscription

    You need to login via Azure CLI first, use azure_login function for this.

    Return:
            dict of Azure AKS cluster and respective resource group if no cluster_name provided.
            
    '''
    # Dict stores Azure AKS cluster and respective Resource group.
    k8slist = {}

    # Temporary array to get the list of AKS cluster from azure.
    tempAKS = []

    list = check_output(get_cmd('k8s_list',cloud),shell=True)

    for a in list.splitlines():

        a = (a.decode('utf-8')).split()
        tempAKS.append(a)

    k8slist = dict(tempAKS)

    if output and k8slist:
        
        if cloud == 'azure':
             header = ['AKS Cluster','Resource Group']
             print(colorama.Fore.GREEN + 'List of AKS clusters which are currently available for your default subscription:')
        else:
            header = ['GKE Cluster','Project Name']
            print(colorama.Fore.GREEN + 'List of GKE clusters which are currently available for your default project:')

        
        _print_table(k8slist,header)

    elif output and not k8slist:
        
        print(colorama.Fore.YELLOW + 'Currently there are no K8s clusters in default project/subscription!!')
        
    else:  
        return k8slist



def addConfig(spinner,cloud,cluster_name):
    
    ''' 
    To add the AKS/GKE cluster configuration to default Kubeasy directory.

    Validate the correct clustername by comparing with the list of AKS cluster for the default Azure subscription

    Arguments: 
            spinner obj
            name of AKS/GKE cluster
    Return:
            'success' if configuration copied successfully.
            'error' while copying the file.
            'incorrect_cluster' if cluster is not present in Azure subscription.
    '''

    k8sCluster = get_K8SList(cloud)

    if cluster_name in k8sCluster:
            
        process = Popen(shlex.split(get_credentials(cloud,cluster_name,k8sCluster[cluster_name])), stdout=PIPE, stderr=PIPE)
        process.communicate()    # execute it, the STDOUT and STDERR are disabled
        rc = process.wait()
        if not rc:
            spinner.succeed(colorama.Fore.GREEN + 'Added "{}" configuration to kubeasy default directory.'.format(cluster_name))
        else:
            spinner.fail(colorama.Fore.RED + 'Error downloading the configuration for the {},Please try to get the config file with the mentioned command --> \"{}\"'.format(cluster_name,get_credentials(cloud,cluster_name,k8sCluster[cluster_name])))

    else:
        
        spinner.fail(colorama.Fore.RED + 'Invalid K8s cluster {} !!'.format(cluster_name))
        get_K8SList(cloud,output=True)




def get_kubeasyList(output=False):
    

    kubeasyList = {}
    config = _config_loader()

    if config:

            for cluster in config.get('clusters', []):

                if cluster['name'] == get_current_context():
                    kubeasyList['** ' + cluster['name']] = cluster['cluster']['server']
                else:
                    kubeasyList['   ' + cluster['name']] = cluster['cluster']['server']

    if output and kubeasyList:
    
        header = ['K8s Cluster','Master']
        print('\n List of clusters which are currently ready to use for kubeasy:')
        print(colorama.Fore.GREEN + '\n - \'kubeasy -d\' to access Kubernetes dashboard for the current context.')
        print(colorama.Fore.GREEN + ' - \'kubeasy -c <cluster_name>\' to switch to another listed context.\n')
        _print_table(kubeasyList,header)
        print(colorama.Fore.GREEN + '\nNote: ** indicates current context.\n')
        
    elif output and not kubeasyList:
        print(colorama.Fore.YELLOW + 'Currently there are no clusters configured for kubeasy, Please check \"kubeasy -h\" for how to add new AKS\\GKE clusters.')
    
    else:

        return kubeasyList
    
def get_GKEList(output=False):
    
    '''
    To get the list of AKS Cluster for the default Azure subscription

    You need to login via Azure CLI first, use azure_login function for this.

    Return:
            dict of Azure AKS cluster and respective resource group if no cluster_name provided.
            
    '''
    # Dict stores Azure AKS cluster and respective Resource group.
    gkeList = {}

    # Temporary array to get the list of AKS cluster from azure.
    tempGKE = []

    list = check_output(get_cmd('k8s_list','google'),shell=True)

    for a in list.splitlines():

        a = (a.decode('utf-8')).split()
        tempGKE.append(a)

    gkeList = dict(tempGKE)

    if output and gkeList:
        
        header = ['GKE Cluster','PROJECT NAME']
        print(colorama.Fore.GREEN + 'List of AKS clusters which are currently available for your default subscription:\n')
        _print_table(gkeList,header)

    elif output and not gkeList:
        
        print(colorama.Fore.YELLOW + 'Currently there are no GKE clusters in default project!!')

    else:  
        return gkeList

def _print_table(list,header):
     
    '''
    Print list in Table format
    Arguments:
              list, or dict
              header which defines the table header
    '''
    x = PrettyTable()
    x.field_names = header
    for item in list:
        x.add_row([item,list[item]])

    print(x)
  

def set_k8s_context(cluster):
    

    config = _config_loader()

    if config:

        if _isExist(cluster) and (cluster != get_current_context()):

            config['current-context'] = cluster

            with open(K8S_CONFIG, 'w+') as stream:

                yaml.safe_dump(config, stream, default_flow_style=False)
            
            print(colorama.Fore.GREEN + '\"{}\" set as the current context.'.format(cluster))
        
        elif _isExist(cluster) and (cluster == get_current_context()):

            print(colorama.Fore.YELLOW + '\"{}\" already set as the current context.'.format(cluster))
        
        else:
            print(colorama.Fore.RED + '\n This is not valid cluster--> {} !!'.format(cluster))
            get_kubeasyList(output=True)
    
    else:

        print(colorama.Fore.RED + '{} is not a valid kubeconfig!'.format(K8S_CONFIG))



##Copied from Azure CLI for AKS

def wait_then_open(url):
    
    """
    Waits for a bit then opens a URL.  Useful for waiting for a proxy to come up, and then open the URL.
    """
    spinner = Halo(text=colorama.Fore.GREEN + 'Opening Kubernetes Dashboard for {} --> {}'.format(get_current_context(),url), spinner='dots',color='yellow')
    spinner.start()
    time.sleep(3)
    webbrowser.open_new_tab(url)
    spinner.info(colorama.Fore.GREEN + 'Running dashboard for {}, Press CTRL+C to stop the port forwarding..'.format(get_current_context()))


##Copied from Azure CLI for AKS

def wait_then_open_async(url):
    """
    Spawns a thread that waits for a bit then opens a URL.
    """
    t = threading.Thread(target=wait_then_open, args=({url}))
    t.daemon = True
    t.start()


def get_dashboard():

    # Get free port 
    with socket() as s:
        s.bind(('',0))
        listen_port = (s.getsockname()[1])
    try:
        pod = (check_output(get_cmd('dashboard_pod'),shell=True).decode('utf-8')).rstrip()

    except subprocess.CalledProcessError as e:
        raise('Not able to find the k8s dashboard pod: {}'.format(e))

    wait_then_open_async('http://localhost:{}'.format(listen_port))
   # wait_then_open_async('http://localhost:{}/api/v1/namespaces/kube-system/services/http:kubernetes-dashboard:/proxy/#!/overview?namespace=default'.format(listen_port))

    try:
        subprocess.call(["kubectl", "-n", "kube-system",
                         "port-forward", pod, "{}:9090".format(listen_port)],stdout=PIPE, stderr=PIPE)
        
    except KeyboardInterrupt:
        return



#!/usr/bin/env python
import sys
import colorama
import click
from halo import Halo
from subprocess import Popen, PIPE, check_output
from config._cmds import get_cmd, get_credentials
from core.configurator import login, get_K8SList, get_GKEList, addConfig, get_kubeasyList, get_dashboard, _isExist, set_k8s_context, get_current_context, get_k8s_config


def get_list(ctx, param, value):

    """ To get list of Kubernetes Clusters (As per ~/.kube/config) """

    if not value or ctx.resilient_parsing:
        return
    get_kubeasyList(output=True)
    ctx.exit()


def set_context(ctx, param, value):

    """ To set the kubernentes cofing context """

    if not value or ctx.resilient_parsing:
        return
    set_k8s_context(value)
    ctx.exit()


def print_version(ctx, param, value):

    """ Get the version of kubeasy """

    if not value or ctx.resilient_parsing:
        return
    click.echo('Kubeasy Version 0.6')
    ctx.exit()


def open_dashboard(ctx, param, value):

    """ Open kubernetes Dashboard with browser tab"""

    if not value or ctx.resilient_parsing:
        return
    
    get_dashboard()
    ctx.exit()



@click.group()
@click.help_option('-h','--help', help="Show the usage of kubeasy.")
@click.option('-v','--version',help="Show the version of kubeasy", is_flag=True, callback=print_version,
              expose_value=False, is_eager=True)
@click.option('-l','--list',is_flag=True,help="List existing clusters in kubeasy.",callback=get_list,expose_value=False, is_eager=False)
@click.option('-c','--context',help="Change k8s config context",callback=set_context,expose_value=False, is_eager=False)
@click.option('-d','--dashboard',is_flag=True,help="Opens up the dashboard for current context",callback=open_dashboard,expose_value=False, is_eager=False)
def cli():

    '''
    
    \b
     _          _   
    | | ___   _| |__   ___  __ _ ___ _   _
    | |/ / | | | '_ \ / _ \/ _` / __| | | |
    |   <| |_| | |_) |  __/ (_| \__ \ |_| |
    |_|\_\\__,_|_.__/ \___|\__,_|___/\__, |
                                      |___/
    \b
    Kubeasy is just an easiest and fastest way to switch between multiple K8's clusters.
    '''



@cli.group()
@click.help_option('-h','--help', help="Show the usage of aks command.")
def aks():

    '''
    \b
    Manages k8s clusters from AKS.

    \b
    You can add new k8s clusters just specifying name of the AKS Cluster,
    Or all the clusters from specific Azure Subscription.

    '''

@aks.command('add', short_help='Add new cluster to kubeasy')
@click.option('-n','--name',required=True,help="Add new kube cluster in kubeasy")
@click.option('-f','--force',required=False,is_flag=True,default=False,help="Forcefully add cluster configuration even if it exists")
@click.help_option('-h','--help', help="Show the usage of add command.")
def add_aks(name,force):
    

    '''
    \b
    Example:
    \b
    # Add all AKS clusters from Azure Subscription.
    kubeasy aks add -n all

    \b
    # Add specific AKS clusters from Azure Subscription.
    kubeasy aks add -n <aksCluster>

    '''
   

    spinner = Halo(text=colorama.Fore.GREEN + 'Logging into Azure using Azure CLI..', spinner='dots',color='yellow')
    spinner.start()

    if not login('azure',spinner):
        spinner.fail(colorama.Fore.RED + 'Azure login failed')
        sys.exit(1)

    spinner.stop()

    spinner = Halo(text=colorama.Fore.GREEN + 'Getting Kubernetes Configuration for {}'.format(name), spinner='dots',color='yellow')
    spinner.start()

    if name == 'all':
                 
        for key in get_K8SList('azure'):
         
             if (not _isExist(key) or (_isExist(key) and force)):
                 
                 addConfig(spinner,'azure',key)
             else:
                 spinner.info(colorama.Fore.GREEN + '\"{}\" is already configured for the Kubeasy, Cheers ! '.format(key))

    elif (not _isExist(name) or (_isExist(name) and force)):
        
        addConfig(spinner,'azure',name)

    else:
        
        spinner.info(colorama.Fore.GREEN + '\"{}\" is already configured for the Kubeasy, Cheers !'.format(name))

    spinner.stop()
    


@cli.group()
@click.help_option('-h','--help', help="Show the usage of gke command.")
def gke():
    
    '''
    \b
    Manages k8s clusters from GKE.

    \b
    You can add new k8s clusters just specifying name of the GKE Cluster

    '''
    


@gke.command('add', short_help='Add new cluster to kubeasy')
@click.option('-n','--name',required=True,help="Add new kube cluster in kubeasy")
@click.option('-f','--force',required=False,is_flag=True,default=False,help="Forcefully add cluster configuration even if it exists")
@click.help_option('-h','--help', help="Show the usage of add command.")
def add_gke(name,force):
    
    '''
    \b
    Example:
    \b
    # Add all GKE clusters .
    kubeasy gke add -n all

    \b
    # Add specific GKE clusters.
    kubeasy gke add -n <gkeCluster>
    '''

    spinner = Halo(text=colorama.Fore.GREEN + 'Logging into Google Cloud using gcloud ..', spinner='dots',color='yellow')
    spinner.start()

    if not login('google',spinner):
        spinner.fail(colorama.Fore.RED + 'Google Cloud login failed')
        sys.exit(1)
    
    spinner.stop()

    spinner = Halo(text=colorama.Fore.GREEN + 'Getting Kubernetes Configuration for {}'.format(name), spinner='dots',color='yellow')
    
    spinner.start()

    if name == 'all':
                 
        for key in get_K8SList('google'):
         
             if (not _isExist(key) or (_isExist(key) and force)):
                 
                 addConfig(spinner,'google',key)
             else:
                 spinner.info(colorama.Fore.GREEN + '\"{}\" is already configured for the Kubeasy, Cheers ! '.format(key))

    elif (not _isExist(name) or (_isExist(name) and force)):
        
        addConfig(spinner,'google',name)

    else:
        
        spinner.info(colorama.Fore.GREEN + '\"{}\" is already configured for the Kubeasy, Cheers !'.format(name))

    spinner.stop()


if __name__ == '__main__':
    cli()


import os
from setuptools import setup, find_packages

DEPENDENCIES = [
    'click',
    'colorama==0.3.9',
    'halo',
    'PyYAML',
    'PTable',
    'pylint',
    'kubernetes'

]

setup(
    name='kubeasy',
    version='0.7',
    py_modules=['kubeasy'],
    python_requires='>=3',
    packages=find_packages(),
    author='Cizer Pereira',
    author_email='cizer.ciz@gmail.com',
    url='https://github.com/C123R/kubeasy',
    include_package_data=True,
    install_requires=DEPENDENCIES,
    entry_points='''
        [console_scripts]
        kubeasy=kubeasy:cli
    ''',
)



import os

def get_k8s_config():

    """ Return kube config file path and create one if not exist """

    k8sConfig = os.path.join(os.path.expanduser('~'), '.kube/config')

    if not os.path.exists(k8sConfig):
        
        open(k8sConfig, 'a').close()

    return k8sConfig